/******************************************************************************/
/* AGENT2B                                                                    */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome:                                                                      */
/*                                                                            */
/* Descrio:                                                                 */
/*    Inclui um fornecedor na planilha de cotao                             */
/*                                                                            */
/* Criao:                                                                   */
/*   03/04/2004 - Joo Fernando dos Reis Corra                               */
/*                                                                            */
/******************************************************************************/

IF EXISTS (SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'INC_FORN_COTA'  and
            type = 'P'                and
            u.uid = o.uid             and
            u.name = USER)
   DROP PROCEDURE INC_FORN_COTA;
GO

CREATE PROCEDURE INC_FORN_COTA
(
   @P_CDFILIAL    VARCHAR(4),
   @P_NRCOLEPRECO VARCHAR(10),
   @P_CDFORNECED  VARCHAR(21),
   @P_CDOPERSIS   VARCHAR(12),
   @P_TREGIAO     VARCHAR(1),
   @P_CDFORNCARG  VARCHAR(21),
   @P_CDREGCARG   VARCHAR(3),
   @P_IDERRO      VARCHAR(3) OUTPUT
)
/*
P_IDERRO: POS   ERRO
          1     REFOREG
          2     QFOPRODFORN
          3     PRODFORNCOMP
*/
AS
   DECLARE @V_CDEMPRESA  VARCHAR(2);
   DECLARE @V_CDINSCMUNI VARCHAR(16);
   DECLARE @V_CDINSCESTA VARCHAR(16);

   DECLARE @V_PGTO_CDPRODUTO    VARCHAR(20);
   DECLARE @V_PGTO_CDFORMPGTO   VARCHAR(2);
   DECLARE @V_PGTO_IDTPFORPGPRF VARCHAR(1);
   DECLARE @V_PGTO_NRPARCPGTPRF NUMERIC(2);
   DECLARE @V_PGTO_NRDIAPGFIPRF NUMERIC(2);
   DECLARE @V_PGTO_IDUTDIAPGPRF VARCHAR(1);

   DECLARE @V_PGTO_IDTPFORPGTFO VARCHAR(1);
   DECLARE @V_PGTO_NRPARCPGTOFO NUMERIC(2);
   DECLARE @V_PGTO_NRDIAPGFIXFO NUMERIC(2);
   DECLARE @V_PGTO_IDUTDIAPGTFO VARCHAR(1);
   DECLARE @V_PGTO_IDTPDESCFORN VARCHAR(1);
   DECLARE @V_PGTO_VRPEDESCFORN DOUBLE PRECISION;
   DECLARE @V_PGTO_IDTPFRETFORN VARCHAR(1);

   /* Variveis para o fetch do cursor c_inclui */
   DECLARE @V_INCLUI_CDREGIAO      VARCHAR(3);
   DECLARE @V_INCLUI_CDPRODUTO     VARCHAR(10);
   DECLARE @V_INCLUI_CDMARCA       VARCHAR(5);
   DECLARE @V_INCLUI_VRPRECUNIITM  DOUBLE PRECISION;
   DECLARE @V_INCLUI_DTINIVALCOL   DATETIME;
   DECLARE @V_INCLUI_DTFINVALCOL   DATETIME;
   DECLARE @V_INCLUI_VRPREDESITM   DOUBLE PRECISION;
   DECLARE @V_INCLUI_IDTPFRETFORN  VARCHAR(1);
   DECLARE @V_INCLUI_CDARVPROD     VARCHAR(200);
   DECLARE @V_INCLUI_DTCOLEPREC    DATETIME;

   DECLARE @V_COUNT         FLOAT;
   DECLARE @V_ROWS_AFFECTED FLOAT;
BEGIN
   SET @P_IDERRO = '000';
   SET @V_ROWS_AFFECTED = 0;

   DECLARE C_INCLUI CURSOR FOR
    SELECT
       ITEMCOLE.CDREGIAO,
       ITEMCOLE.CDPRODUTO,
       ITEMCOLE.CDMARCA,
       ISNULL(CARGPRECO.VRPRECUNIITM, 0) AS VRPRECUNIITM,
       COLETAPR.DTINIVALCOL,
       COLETAPR.DTFINVALCOL,

       CASE FORNECEDOR.IDTPDESCFORN
         WHEN 'C' THEN FORNECEDOR.VRPEDESCFORN
         ELSE 0
       END AS VRPREDESITM,

       FORNECEDOR.IDTPFRETFORN,
       PRODUTO.CDARVPROD,
       COLETAPR.DTCOLEPREC
    FROM
       (SELECT
           REGIAO.CDREGIAO,
           ITEMCOLE.CDPRODUTO,
           ITEMCOLE.CDMARCA
        FROM
           (SELECT DISTINCT
               CDPRODUTO,
               CDMARCA
            FROM
               ITEMCOLE
            WHERE
               CDFILIAL    = @P_CDFILIAL AND
               NRCOLEPRECO = @P_NRCOLEPRECO AND
               CDFORNECED  = ISNULL(@P_CDFORNCARG, CDFORNECED) AND
               CDREGIAO    = ISNULL(@P_CDREGCARG,  CDREGIAO)
           ) ITEMCOLE,

           (SELECT
              REGIAO.CDREGIAO
            FROM
              (SELECT
                  CDREGIAO
               FROM
                  REGIAO
               WHERE
                  @P_TREGIAO = 'S'
               UNION ALL
               SELECT DISTINCT
                  CDREGIAO
               FROM
                  ITEMCOLE
               WHERE
                  CDFILIAL    = @P_CDFILIAL AND
                  NRCOLEPRECO = @P_NRCOLEPRECO AND
                  @P_TREGIAO = 'N') REGIAO
           ) REGIAO
        ) ITEMCOLE
        LEFT JOIN
           (SELECT
               CDREGIAO,
               CDPRODUTO,
               CDMARCA,
               VRPRECUNIITM
            FROM
               ITEMCOLE
            WHERE
               CDFILIAL    = @P_CDFILIAL    AND
               NRCOLEPRECO = @P_NRCOLEPRECO AND
               CDFORNECED  = @P_CDFORNCARG
           ) CARGPRECO
        ON
         ISNULL(@P_CDREGCARG, ITEMCOLE.CDREGIAO) = CARGPRECO.CDREGIAO AND
         ITEMCOLE.CDPRODUTO = CARGPRECO.CDPRODUTO AND
         ITEMCOLE.CDMARCA   = CARGPRECO.CDMARCA,

       FORNECEDOR,
       COLETAPR,
       PRODUTO
    WHERE
       COLETAPR.CDFILIAL     = @P_CDFILIAL    AND
       COLETAPR.NRCOLEPRECO  = @P_NRCOLEPRECO AND
       FORNECEDOR.CDFORNECED = @P_CDFORNECED  AND
       PRODUTO.CDPRODUTO     = ITEMCOLE.CDPRODUTO;

   OPEN C_INCLUI;
   FETCH NEXT FROM C_INCLUI INTO @V_INCLUI_CDREGIAO,
                                 @V_INCLUI_CDPRODUTO,
                                 @V_INCLUI_CDMARCA,
                                 @V_INCLUI_VRPRECUNIITM,
                                 @V_INCLUI_DTINIVALCOL,
                                 @V_INCLUI_DTFINVALCOL,
                                 @V_INCLUI_VRPREDESITM,
                                 @V_INCLUI_IDTPFRETFORN,
                                 @V_INCLUI_CDARVPROD,
                                 @V_INCLUI_DTCOLEPREC;


   WHILE (@@FETCH_STATUS = 0)  BEGIN

      SET @V_COUNT = 0;
      SELECT
        @V_COUNT = COUNT(*)
      FROM
        REFOREG
      WHERE
        REFOREG.CDFORNECED = @P_CDFORNECED AND
        REFOREG.CDREGIAO   = @V_INCLUI_CDREGIAO;

      IF @V_COUNT > 0 BEGIN

         SET @P_IDERRO = '1' + SUBSTRING(@P_IDERRO, 2, 2);

      END
      ELSE BEGIN
         SET @V_COUNT = 0;
         SELECT
            @V_COUNT = COUNT(*)
         FROM
            QFOPRODFORN,
            PARASUPRI
         WHERE
           PARASUPRI.CDFILIAL = @P_CDFILIAL  AND
           (QFOPRODFORN.CDPRODUTO = @V_INCLUI_CDPRODUTO) AND
           (QFOPRODFORN.CDFORNECED= @P_CDFORNECED) AND
           ((QFOPRODFORN.IDFORNAPROV = 'N' AND QFOPRODFORN.IDFORNAUTOR = 'N') OR
            ((QFOPRODFORN.IDFORNAPROV = 'S') AND
               ((PARASUPRI.IDULTIDTVALQ = 'M' AND
                 @V_INCLUI_DTCOLEPREC > (DATEADD(MM,1,QFOPRODFORN.DTVALPRFORN)- DAY(DATEADD(MM,1,QFOPRODFORN.DTVALPRFORN)))) OR
                (PARASUPRI.IDULTIDTVALQ = 'D' AND
                 @V_INCLUI_DTCOLEPREC > QFOPRODFORN.DTVALPRFORN)) ) );

         IF @V_COUNT > 0 BEGIN

            SET @P_IDERRO = SUBSTRING(@P_IDERRO, 1, 1) + '1' + SUBSTRING(@P_IDERRO, 3, 1);

         END
         ELSE BEGIN

            SET @V_COUNT = 0;
            SELECT
              @V_COUNT = COUNT(*)
            FROM
              PRODFORNCOMP
            WHERE
              PRODFORNCOMP.CDFORNECED = @P_CDFORNECED  AND
              PRODFORNCOMP.CDPRODCOMPL = @V_INCLUI_CDPRODUTO;

            IF @V_COUNT = 0 BEGIN

               SET @P_IDERRO = SUBSTRING(@P_IDERRO, 1, 2) + '1';

            END
            ELSE BEGIN

               SET @V_ROWS_AFFECTED = @V_ROWS_AFFECTED + 1;

               INSERT INTO ITEMCOLE
                  (CDFILIAL,     NRCOLEPRECO,  CDFORNECED,   CDREGIAO,
                   CDPRODUTO,    CDMARCA,      VRPRECUNIITM, DTINIVALITM,
                   DTFINVALITM,  VRPERDESITM,  IDTIPFREITM,  IDITMCOTPREC,
                   CDOPERINCL,   DTINCLITMCOL)
               VALUES
                  (@P_CDFILIAL,           @P_NRCOLEPRECO,        @P_CDFORNECED,          @V_INCLUI_CDREGIAO,
                   @V_INCLUI_CDPRODUTO,   @V_INCLUI_CDMARCA,     @V_INCLUI_VRPRECUNIITM, @V_INCLUI_DTINIVALCOL,
                   @V_INCLUI_DTFINVALCOL, @V_INCLUI_VRPREDESITM, @V_INCLUI_IDTPFRETFORN,  'S',
                   @P_CDOPERSIS,          GETDATE());

               IF @V_INCLUI_VRPRECUNIITM > 0 BEGIN

         /*###############################  IMPOSTO ######################################*/
                 SET @V_CDEMPRESA = NULL;
                 SELECT
                    @V_CDEMPRESA = MAX(EMITENTEAF.CDEMPRESA)
                 FROM
                    EMITENTEAF, PRODUTO
                 WHERE
                    EMITENTEAF.CDPRODUTO = PRODUTO.CDPRODUTO AND
                    EMITENTEAF.CDFILIAL = @P_CDFILIAL AND
                    @V_INCLUI_CDARVPROD LIKE PRODUTO.CDARVPROD + '%';

                 IF @V_CDEMPRESA IS NULL BEGIN

                    SET @V_CDEMPRESA = NULL;
                    SELECT
                       @V_CDEMPRESA = MAX(PARCMPSUP.CDEMPRESA)
                    FROM
                       PARCMPSUP
                    WHERE
                       PARCMPSUP.CDFILIAL = @P_CDFILIAL;

                 END;
                 SET @V_CDEMPRESA  = NULL;
                 SET @V_CDINSCMUNI = NULL;
                 SET @V_CDINSCESTA = NULL;
                 SELECT
                    @V_CDEMPRESA  = MAX(EMPRESA.CDEMPRESA),
                    @V_CDINSCMUNI = MIN(INSCRMUNICIP.CDINSCMUNI),
                    @V_CDINSCESTA = MIN(INSCRESTAD.CDINSCESTA)
                 FROM
                    EMPRESA,
                    INSCRMUNICIP,
                    INSCRESTAD
                 WHERE
                    EMPRESA.CDEMPRESA = @V_CDEMPRESA           AND
                    EMPRESA.CDEMPRESA = INSCRMUNICIP.CDEMPRESA AND
                    EMPRESA.CDEMPRESA = INSCRESTAD.CDEMPRESA;

                 EXEC IMPOSTO_ITEMCOL @V_CDEMPRESA,      @V_CDINSCESTA, @P_CDFORNECED,      @P_CDFILIAL,
                                      @P_NRCOLEPRECO,    @P_CDFORNECED, @V_INCLUI_CDREGIAO, @V_INCLUI_CDPRODUTO,
                                      @V_INCLUI_CDMARCA, @V_INCLUI_DTCOLEPREC;

               /*############################# FORMA DE PAGAMENTO ##############################*/
                 SET @V_PGTO_CDPRODUTO    = NULL;
                 SET @V_PGTO_CDFORMPGTO   = NULL;
                 SET @V_PGTO_IDTPFORPGPRF = NULL;
                 SET @V_PGTO_NRPARCPGTPRF = NULL;
                 SET @V_PGTO_NRDIAPGFIPRF = NULL;
                 SET @V_PGTO_IDUTDIAPGPRF = NULL;
                 SELECT
                    @V_PGTO_CDPRODUTO    = MAX(PRODUTO.CDARVPROD),
                    @V_PGTO_CDFORMPGTO   = MAX(PRODFORN.CDFORMPGTO),
                    @V_PGTO_IDTPFORPGPRF = MAX(PRODFORN.IDTPFORPGPRF),
                    @V_PGTO_NRPARCPGTPRF = MAX(PRODFORN.NRPARCPGTPRF),
                    @V_PGTO_NRDIAPGFIPRF = MAX(PRODFORN.NRDIAPGFIPRF),
                    @V_PGTO_IDUTDIAPGPRF = MAX(PRODFORN.IDUTDIAPGPRF)
                 FROM
                   PRODFORN, PRODUTO
                 WHERE
                   PRODFORN.CDPRODUTO  = PRODUTO.CDPRODUTO     AND
                   PRODFORN.CDFORNECED = @P_CDFORNECED AND
                   PRODUTO.CDARVPROD   = (SELECT MAX(PRODUTO.CDARVPROD)
                                          FROM PRODFORN, PRODUTO
                                          WHERE
                                          PRODFORN.CDPRODUTO   = PRODUTO.CDPRODUTO AND
                                          PRODFORN.CDFORNECED  = @P_CDFORNECED AND
                                          @V_INCLUI_CDARVPROD LIKE PRODUTO.CDARVPROD + '%');

                 IF @V_PGTO_IDTPFORPGPRF IS NULL BEGIN
                    SET @V_PGTO_CDFORMPGTO   = NULL;
                    SET @V_PGTO_IDTPFORPGTFO = NULL;
                    SET @V_PGTO_NRPARCPGTOFO = NULL;
                    SET @V_PGTO_NRDIAPGFIXFO = NULL;
                    SET @V_PGTO_IDUTDIAPGTFO = NULL;
                    SET @V_PGTO_IDTPDESCFORN = NULL;
                    SET @V_PGTO_VRPEDESCFORN = NULL;
                    SET @V_PGTO_IDTPFRETFORN = NULL;
                    SELECT
                      @V_PGTO_CDFORMPGTO   = CDFORMPGTO,
                      @V_PGTO_IDTPFORPGTFO = IDTPFORPGTFO,
                      @V_PGTO_NRPARCPGTOFO = NRPARCPGTOFO,
                      @V_PGTO_NRDIAPGFIXFO = NRDIAPGFIXFO,
                      @V_PGTO_IDUTDIAPGTFO = IDUTDIAPGTFO,
                      @V_PGTO_IDTPDESCFORN = IDTPDESCFORN,
                      @V_PGTO_VRPEDESCFORN = VRPEDESCFORN,
                      @V_PGTO_IDTPFRETFORN = IDTPFRETFORN
                    FROM
                       FORNECEDOR
                    WHERE
                       CDFORNECED = @P_CDFORNECED;

                    UPDATE ITEMCOLE
                    SET
                       CDFORMPGTO   = @V_PGTO_CDFORMPGTO,
                       IDTPFOPGTCOL = @V_PGTO_IDTPFORPGTFO,
                       NRDIAPGFXCOL = @V_PGTO_NRDIAPGFIXFO,
                       NRPARCPGTCOL = @V_PGTO_NRPARCPGTOFO,
                       IDUTDIAPGCOL = @V_PGTO_IDUTDIAPGTFO
                    WHERE
                       CDFILIAL    = @P_CDFILIAL   AND
                       NRCOLEPRECO = @P_NRCOLEPRECO AND
                       CDPRODUTO   = @V_INCLUI_CDPRODUTO  AND
                       CDMARCA     = @V_INCLUI_CDMARCA    AND
                       CDFORNECED  = @P_CDFORNECED AND
                       CDREGIAO    = @V_INCLUI_CDREGIAO;

                    IF @V_PGTO_IDTPFORPGTFO = 'V' BEGIN

                       INSERT INTO PZOPGTITMCOL
                         (CDFILIAL, NRCOLEPRECO, CDFORNECED, CDPRODUTO,
                         CDMARCA, CDREGIAO, NRDIASPGTCOL, VRPERCPGTCOL)
                       SELECT
                          @P_CDFILIAL,
                          @P_NRCOLEPRECO,
                          @P_CDFORNECED,
                          @V_INCLUI_CDPRODUTO,
                          @V_INCLUI_CDMARCA,
                          @V_INCLUI_CDREGIAO,
                          PRAZOPGTFORN.NRDIASPGTOFO,
                          PRAZOPGTFORN.VRPERCPGTOFO
                       FROM
                          PRAZOPGTFORN
                       WHERE
                          PRAZOPGTFORN.CDFORNECED = @P_CDFORNECED;

                    END;
                 END
                 ELSE BEGIN

                    UPDATE ITEMCOLE
                    SET
                       CDFORMPGTO   = @V_PGTO_CDFORMPGTO,
                       IDTPFOPGTCOL = @V_PGTO_IDTPFORPGPRF,
                       NRDIAPGFXCOL = @V_PGTO_NRDIAPGFIPRF,
                       NRPARCPGTCOL = @V_PGTO_NRPARCPGTPRF,
                       IDUTDIAPGCOL = @V_PGTO_IDUTDIAPGPRF
                    WHERE
                       CDFILIAL    = @P_CDFILIAL   AND
                       NRCOLEPRECO = @P_NRCOLEPRECO AND
                       CDPRODUTO   = @V_INCLUI_CDPRODUTO  AND
                       CDMARCA     = @V_INCLUI_CDMARCA    AND
                       CDFORNECED  = @P_CDFORNECED AND
                       CDREGIAO    = @V_INCLUI_CDREGIAO;

                    IF @V_PGTO_IDTPFORPGPRF = 'V' BEGIN

                       INSERT INTO PZOPGTITMCOL
                         (CDFILIAL, NRCOLEPRECO, CDFORNECED, CDPRODUTO,
                         CDMARCA, CDREGIAO, NRDIASPGTCOL, VRPERCPGTCOL)

                       SELECT
                          @P_CDFILIAL,
                          @P_NRCOLEPRECO,
                          @P_CDFORNECED,
                          @V_INCLUI_CDPRODUTO,
                          @V_INCLUI_CDMARCA,
                          @V_INCLUI_CDREGIAO,
                          PZOPGTPRFO.NRDIASPGPRFO,
                          PZOPGTPRFO.VRPERCPGPRFO
                       FROM
                          PZOPGTPRFO, PRODUTO
                       WHERE
                          PZOPGTPRFO.CDPRODUTO  = PRODUTO.CDPRODUTO     AND
                          PZOPGTPRFO.CDFORNECED = @P_CDFORNECED AND
                          @V_INCLUI_CDARVPROD LIKE PRODUTO.CDARVPROD + '%';

                    END;

                 END;

         /*############ Forando execuo da trigger para clculo do preo econmico ###############*/
                 UPDATE ITEMCOLE
                 SET IDTPFOPGTCOL = IDTPFOPGTCOL
                 WHERE
                   CDFILIAL    = @P_CDFILIAL    AND
                   NRCOLEPRECO = @P_NRCOLEPRECO AND
                   CDPRODUTO   = @V_INCLUI_CDPRODUTO   AND
                   CDMARCA     = @V_INCLUI_CDMARCA     AND
                   CDFORNECED  = @P_CDFORNECED  AND
                   CDREGIAO    = @V_INCLUI_CDREGIAO;


               END;

            END;

         END;

      END;
      FETCH NEXT FROM C_INCLUI INTO @V_INCLUI_CDREGIAO,
                                    @V_INCLUI_CDPRODUTO,
                                    @V_INCLUI_CDMARCA,
                                    @V_INCLUI_VRPRECUNIITM,
                                    @V_INCLUI_DTINIVALCOL,
                                    @V_INCLUI_DTFINVALCOL,
                                    @V_INCLUI_VRPREDESITM,
                                    @V_INCLUI_IDTPFRETFORN,
                                    @V_INCLUI_CDARVPROD,
                                    @V_INCLUI_DTCOLEPREC;
   END;

   CLOSE      C_INCLUI;
   DEALLOCATE C_INCLUI;

   IF @V_ROWS_AFFECTED <> 0 BEGIN
      IF CHARINDEX('1', @P_IDERRO) <> 0 BEGIN
        IF SUBSTRING(@P_IDERRO, 1, 1) = 1 BEGIN
          SET @P_IDERRO = '2' + SUBSTRING(@P_IDERRO, 2, 2);
        END;
        IF SUBSTRING(@P_IDERRO, 2, 1) = 1 BEGIN
          SET @P_IDERRO = SUBSTRING(@P_IDERRO, 1, 1) + '2' + SUBSTRING(@P_IDERRO, 3, 1);
        END;
        IF SUBSTRING(@P_IDERRO, 3, 1) = 1 BEGIN
          SET @P_IDERRO = SUBSTRING(@P_IDERRO, 1, 2) + '2';
        END;
      ELSE
        SET @P_IDERRO = '000';
      END;
   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'INC_FORN_COTA','6.19.XXX';
GO
